import pytest
from django.core.management import call_command, CommandError


@pytest.mark.django_db
class TestCommand(object):
    def test_command_output(self, capsys):
        with capsys.disabled():
            call_command('migrate', 'test_app', '0001')

        with pytest.raises(CommandError, match='WTF!!!!!!!!!!! Migrations are not secure.'):
            call_command('check_migration')
        captured = capsys.readouterr()

        assert 'SQL is not secure to do without downtime' in captured.err

    def test_command_output_not_sql_insecure(self, capsys):
        with capsys.disabled():
            call_command('migrate', 'test_app', '0002')
        call_command('check_migration')
        captured = capsys.readouterr()

        assert not captured.err

    def test_command_github(self, capsys):
        with capsys.disabled():
            call_command('migrate', 'test_app_github', '0001')

        with pytest.raises(CommandError, match='WTF!!!!!!!!!!! Migrations are not secure.'):
            call_command('check_migration', '--github', 'f6d560cb06b54d7f6f8ec94cf9a510632aed25fc', 'test-github-integration-without-migration-not-delete')
        captured = capsys.readouterr()

        assert 'SQL is not secure to do without downtime' in captured.err



# Generated by Django 2.2.7 on 2020-04-26 23:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0002_auto_20200426_1246'),
    ]

    operations = [
        migrations.CreateModel(
            name='Truck',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('size', models.IntegerField(db_index=True)),
            ],
        ),
    ]



from unittest import mock

import pytest

from django_check_migration_wtf.rules import AbstractSQLRule
from django_check_migration_wtf import psql


class TestAbstractSQLRule(object):
    @pytest.mark.parametrize('sql_sentence,expected',
                             [('test', True),
                              ('TEST', True),
                              ('bad', False)]
                             )
    @mock.patch.multiple('django_check_migration_wtf.rules.AbstractSQLRule', __abstractmethods__=set())
    @mock.patch('django_check_migration_wtf.rules.AbstractSQLRule.pattern',
                new_callable=mock.PropertyMock(return_value='test')
                )
    def test_is_match_true(self, _, sql_sentence, expected):
        rule = AbstractSQLRule()
        assert rule.is_match(sql_sentence) == expected


class TestRenameColumnSQLRule:
    def test_is_match(self):
        rule = psql.RenameColumnSQLRule()

        assert rule.is_match('ALTER TABLE test RENAME COLUMN column_a TO column_b;')


class TestRenameTableSQLRule:
    def test_is_match(self):
        rule = psql.RenameTableSQLRule()

        assert rule.is_match('ALTER TABLE test RENAME TO column_b;')


class TestAddColumnNotNullSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD COLUMN column_test double precision DEFAULT 1.0 NOT NULL;',
        'ALTER TABLE test ADD COLUMN column_test double precision NOT NULL DEFAULT 1.0;',
        'ALTER TABLE test ADD COLUMN column_test integer NOT NULL;',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AddColumnNotNullSQLRule()

        assert rule.is_match(sql_sentence)


class TestAddColumnDefaultSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD COLUMN column_test double precision DEFAULT 1.0 NOT NULL;',
        'ALTER TABLE test ADD COLUMN column_test double precision NOT NULL DEFAULT 1.0;',
        'ALTER TABLE test ADD COLUMN column_test integer DEFAULT 1;',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AddColumnDefaultSQLRule()

        assert rule.is_match(sql_sentence)


class TestAddColumnPrimaryKeySQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD COLUMN column_test integer DEFAULT 1 PRIMARY KEY NOT NULL;',
        'ALTER TABLE test ADD COLUMN column_test double precision NOT NULL PRIMARY KEY;',
        'ALTER TABLE test ADD COLUMN column_test double precision PRIMARY KEY;',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AddColumnPrimaryKeySQLRule()

        assert rule.is_match(sql_sentence)


class TestAddColumnUniqueSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD COLUMN column_test integer DEFAULT 1 UNIQUE NOT NULL;',
        'ALTER TABLE test ADD COLUMN column_test double precision NOT NULL UNIQUE;',
        'ALTER TABLE test ADD COLUMN column_test double precision UNIQUE;',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AddColumnUniqueSQLRule()

        assert rule.is_match(sql_sentence)


class TestAlterColumnTypeSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ALTER COLUMN column_test TYPE integer;',
        'ALTER TABLE test ALTER COLUMN column_test SET DATA TYPE integer;',
        'ALTER TABLE test ALTER COLUMN column_test TYPE varchar(11) USING a::varchar(11);',
        'ALTER TABLE test ALTER COLUMN column_test SET DATA TYPE varchar(11) USING a::varchar(11);',
        'ALTER TABLE test ALTER COLUMN column_test SET DATA TYPE varchar(11) USING a::varchar(11), ALTER COLUMN "column_test" DROP NOT NULL;',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AlterColumnTypeSQLRule()

        assert rule.is_match(sql_sentence)


class TestAlterColumnAddConstraintSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD CONSTRAINT constraint_test CHECK (char_length(column_test) = 5);',
        'ALTER TABLE test ADD CONSTRAINT constraint_test FOREIGN KEY (column_test) REFERENCES column_test (column_test) MATCH FULL;',
        'ALTER TABLE test ADD CONSTRAINT constraint_test UNIQUE (column_test_1, column_test_2);',
        'ALTER TABLE test ADD CONSTRAINT constraint_test PRIMARY KEY (column_test_1);',
        'ALTER TABLE test ADD CONSTRAINT "not valid constraint_test" PRIMARY KEY (column_test_1);',
        'ALTER TABLE test ADD CONSTRAINT "using index constraint_test" PRIMARY KEY (column_test_1);',
        'ALTER TABLE test ADD CONSTRAINT "ADD CONSTRAINT using index constraint_test" PRIMARY KEY (column_test_1);',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AlterColumnAddConstraintSQLRule()

        assert rule.is_match(sql_sentence)

    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD CONSTRAINT constraint_test CHECK (char_length(column_test) = 5) NOT VALID;',
        'ALTER TABLE test ADD CONSTRAINT constraint_test CHECK (char_length(column_test) = 5) NOT VALID ;',
        'ALTER TABLE test ADD CONSTRAINT constraint_test FOREIGN KEY (column_test) REFERENCES column_test (column_test) MATCH FULL NOT VALID;',
        'ALTER TABLE test ADD CONSTRAINT constraint_test PRIMARY KEY USING INDEX index_test;',
        'ALTER TABLE test ADD CONSTRAINT constraint_test UNIQUE USING INDEX index_test ;',
    ])
    def test_not_match(self, sql_sentence):
        rule = psql.AlterColumnAddConstraintSQLRule()

        assert not rule.is_match(sql_sentence)


class TestAlterColumnAddPrimaryKeySQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'ALTER TABLE test ADD PRIMARY KEY (column_test);',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.AlterColumnAddPrimaryKeySQLRule()

        assert rule.is_match(sql_sentence)


class TestCreateIndexSQLRule:
    @pytest.mark.parametrize('sql_sentence', [
        'CREATE INDEX index_test on test (column_test);',
        'CREATE UNIQUE INDEX index_test on test (column_test);',
        'CREATE UNIQUE INDEX concurrently_test on test (column_test);',
        'CREATE UNIQUE INDEX "concurrently test" on test (column_test);',
    ])
    def test_is_match(self, sql_sentence):
        rule = psql.CreateIndexSQLRule()

        assert rule.is_match(sql_sentence)\


    @pytest.mark.parametrize('sql_sentence', [
        'CREATE INDEX CONCURRENTLY index_test on test (column_test);',
        'CREATE UNIQUE INDEX CONCURRENTLY index_test on test (column_test);',
    ])
    def test_is_not_match(self, sql_sentence):
        rule = psql.CreateIndexSQLRule()

        assert not rule.is_match(sql_sentence)


import re
from typing import List, Tuple, Optional

from django.conf import settings

from .exceptions import SQLRuleError
from .rules import AbstractSQLRule
from . import psql


class SQLRuleEvaluator(object):
    def __init__(self):
        self.rules: List[AbstractSQLRule] = []

    def load_psql_rules(self):
        self.rules = [
            psql.RenameTableSQLRule(),
            psql.RenameColumnSQLRule(),
            psql.AddColumnNotNullSQLRule(),
            psql.AddColumnPrimaryKeySQLRule(),
            psql.AddColumnUniqueSQLRule(),
            psql.AlterColumnTypeSQLRule(),
            psql.AlterColumnAddPrimaryKeySQLRule(),
            psql.AlterColumnAddConstraintSQLRule(),
            psql.CreateIndexSQLRule(),
        ]

        psql_version = getattr(settings, 'CHECK_MIGRATION_WTF_PSQL_VERSION', 9)
        if psql_version < 11:
            self.rules.append(psql.AddColumnDefaultSQLRule())

    def evaluate(self, sql_sentence: str):
        for rule in self.rules:
            if rule.is_match(sql_sentence):
                raise SQLRuleError(rule.error_info)


class SQLStatementsEvaluator:
    SQL_COMMENT = '--'
    SQL_CREATE_TABLE = 'create table '

    def __init__(self, sql_statements):
        self.sql_statements = sql_statements
        self.name_create_tables = []
        self.sql_rule_evaluator = SQLRuleEvaluator()
        self.sql_rule_evaluator.load_psql_rules()
        self.create_rex = re.compile(r'^create .*table (if not exists )?(?P<name>(".*"|.*[^\s]*)) \(.*')

    def evaluate(self) -> List[Optional[Tuple[str, SQLRuleError]]]:
        errors = []
        sql_line: str
        for sql_line in self.sql_statements:
            sql_line = sql_line.lower()
            if self.is_comment(sql_line):
                continue
            if self.is_create_table(sql_line):
                self.name_create_tables.append(self.get_name_create_table(sql_line))
            if self.should_skip(sql_line):
                continue
            try:
                self.sql_rule_evaluator.evaluate(sql_line)
            except SQLRuleError as e:
                errors.append((sql_line, e))

        return errors

    def is_comment(self, sql_line: str) -> bool:
        return sql_line.startswith(self.SQL_COMMENT)

    def is_create_table(self, sql_line: str) -> bool:
        return sql_line.startswith(self.SQL_CREATE_TABLE)

    def should_skip(self, sql_line: str) -> bool:
        for name in self.name_create_tables:
            if name in sql_line:
                return True
        return False

    def get_name_create_table(self, sql_line: str) -> str:
        m = self.create_rex.match(sql_line)
        return m.group('name')



from unittest import mock

import pytest

from django_check_migration_wtf.rules import AbstractSQLRule
from django_check_migration_wtf.github_migrations import GithubMigrations


@pytest.fixture(scope='class')
def dummy_sql_rule():
    class DummySQLRule(AbstractSQLRule):

        @property
        def pattern(self) -> str:
            return 'dummy'

        @property
        def error_info(self) -> str:
            return 'Dummy match'

    return DummySQLRule()


@pytest.fixture(scope='function')
@mock.patch('django_check_migration_wtf.github_migrations.Github')
def github_migrations(_):
    return GithubMigrations('commit_hash', 'base_branch')



import re
from abc import ABC, abstractmethod
from typing import Pattern


class AbstractSQLRule(ABC):
    def __init__(self):
        self.regex: Pattern = re.compile(self.pattern)

    @property
    @abstractmethod
    def pattern(self) -> str:
        """Pattern for regex"""

    def is_match(self, sql_sentence: str) -> bool:
        if self.regex.fullmatch(sql_sentence.lower()) is None:
            return False
        else:
            return True

    @property
    @abstractmethod
    def error_info(self) -> str:
        """Give the error info about rule"""



# Generated by Django 2.2.7 on 2020-05-03 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app_github', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='motorbike',
            name='price',
            field=models.FloatField(db_index=True),
        ),
    ]



from .rules import AbstractSQLRule


class RenameColumnSQLRule(AbstractSQLRule):

    @property
    def pattern(self) -> str:
        return r'alter table .* rename column .* to .*;'

    @property
    def error_info(self) -> str:
        return 'You cannot rename a column, it is an blocking operation. ' \
               'You should create a new column and copy their values'


class RenameTableSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* rename to .*;'

    @property
    def error_info(self) -> str:
        return 'You cannot rename a table, it is an blocking operation. ' \
               'You should create a new table and copy their values'


class AddColumnNotNullSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add column .* not null( .*;|;)'

    @property
    def error_info(self) -> str:
        return 'You cannot add column to table with not null. Because old instance will old code will dead. ' \
               'You should add column with nullable'


class AddColumnDefaultSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add column .* default .*;'

    @property
    def error_info(self) -> str:
        return 'You cannot add column to table with default value, ' \
               'It is a blocking operation because all rows will be populate with this value. ' \
               'You should add column without default and then alter column to set default value.'


class AddColumnPrimaryKeySQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add column .* primary key( .*;|;)'

    @property
    def error_info(self) -> str:
        return 'You cannot add column to table with primary key , ' \
               'It is a blocking operation because it will create a index. ' \
               'You should add column without primary key, ' \
               'then create a unique index concurrently and finally add the constraint.'


class AddColumnUniqueSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add column .* unique( .*;|;)'

    @property
    def error_info(self) -> str:
        return 'You cannot add column to table with unique , ' \
               'It is a blocking operation because it will create a index. ' \
               'You should add column without unique, ' \
               'then create a unique index concurrently and finally add the constraint.'


class AlterColumnTypeSQLRule(AbstractSQLRule):
    # TODO add support for types that they are secures
    @property
    def pattern(self) -> str:
        return r'alter table .* alter column .* (set data type|type) .*(,|;).*'

    @property
    def error_info(self) -> str:
        return 'You cannot alter type of column, ' \
               'It is a blocking operation because it will need check all items of column are valid. ' \
               'in some case it can be safe, but this functionality is not support at this moment. ' \
               'You should create a new column and copy the data to it.'


class AlterColumnAddConstraintSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add constraint ((".*").)*((?!not valid( |;))(?!using index ).)*;'

    @property
    def error_info(self) -> str:
        return 'You cannot add constraint without using a index or adding it with not valid, ' \
               'It is a blocking operation because it will need check all items to validate the constraint.'


class AlterColumnAddPrimaryKeySQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'alter table .* add primary key .*;'

    @property
    def error_info(self) -> str:
        return 'You cannot add primary key, ' \
               'It is a blocking operation because it will need create a unique index and validate it.' \
               'You should be create a unique index (concurrently) and then add constraint using index'


class CreateIndexSQLRule(AbstractSQLRule):
    @property
    def pattern(self) -> str:
        return r'create (unique )*index((?! concurrently ).)* on .*;'

    @property
    def error_info(self) -> str:
        return 'You cannot create a index without concurrently. It is a blocking operation.'



import re
from typing import List, Tuple, Optional

from django.conf import settings
from github import Github
from github.Comparison import Comparison
from github.File import File
from github.Repository import Repository


class GithubMigrations:
    FILE_STATUS_ADDED = 'added'
    FILE_STATUS_MODIFIED = 'modified'
    FILE_STATUS = (FILE_STATUS_ADDED, FILE_STATUS_MODIFIED,)

    def __init__(self, commit_hash: str, base_branch: str):
        self.g = Github(settings.CHECK_MIGRATION_WTF_GITHUB_TOKEN)
        self.repo: Repository = self.g.get_repo(settings.CHECK_MIGRATION_WTF_REPO_NAME)
        self.comp: Comparison = self.repo.compare(base_branch, commit_hash)
        self.migration_name_regex = re.compile(r'(?P<path>.*)/migrations/(?P<migration_name>(?!__init__.py).*).py\Z')

    @property
    def migrations(self) -> List[Tuple[str, str]]:
        migrations: List[Tuple[str, str]] = []
        file: File

        for file in self.comp.files:
            migration_name = self.get_migration_name(file.filename)
            if file.status not in self.FILE_STATUS or migration_name is None:
                continue

            migrations.append((file.filename, migration_name))

        return migrations

    def get_migration_name(self, filename: str) -> Optional[str]:
        m = self.migration_name_regex.match(filename)
        return m.group('migration_name') if m else None



from django.core.management.base import BaseCommand, CommandError
from django.db import DEFAULT_DB_ALIAS, connections
from django.db.migrations.executor import MigrationExecutor

from ...evaluator import SQLStatementsEvaluator
from ...github_migrations import GithubMigrations


class Command(BaseCommand):
    help = "Checks migrations without apply to know if they are secure to do without downtime"
    output_transaction = True

    def __init__(self, stdout=None, stderr=None, no_color=False, force_color=False):
        super().__init__(stdout, stderr, no_color, force_color)

        self.connection = connections[DEFAULT_DB_ALIAS]
        self.executor = MigrationExecutor(self.connection)
        self.loader = self.executor.loader
        self.error = False

    def add_arguments(self, parser):
        parser.add_argument(
            'app_label', nargs='*',
            help='App labels of applications to limit the check.',
        )

        parser.add_argument(
            '--github', nargs=2, type=str,
            help='Check migrations comparing hash of commit with name of base branch',
        )

    def handle(self, *args, **options):
        if options.get('github'):
            github_migrations = GithubMigrations(options['github'][0], options['github'][1])
            migrations = github_migrations.migrations
            for filename, migration_name in migrations:
                app_name = self.get_app_name(filename)
                self.evaluate_migration(app_name, migration_name)

        else:
            # Work out which apps have migrations and which do not
            graph = self.executor.loader.graph
            for app_name in self.loader.migrated_apps:
                for node in graph.leaf_nodes(app_name):
                    for plan_node in graph.forwards_plan(node):
                        # Mark it as applied/unapplied
                        if plan_node not in self.loader.applied_migrations:
                            self.evaluate_migration(plan_node[0], plan_node[1])

        if self.error:
            raise CommandError('WTF!!!!!!!!!!! Migrations are not secure.')

        self.stdout.write(self.style.SUCCESS('Migrations are secure.'))

    def evaluate_migration(self, app_label, migration_name):
        migration = self.loader.get_migration_by_prefix(app_label, migration_name)
        self.output_transaction = migration.atomic and self.connection.features.can_rollback_ddl

        plan = self.loader.graph.nodes[app_label, migration_name]
        if hasattr(self.executor, 'collect_sql'):
            # support django <3.1
            sql_statements = self.executor.collect_sql([(plan, False)])
        else:
            sql_statements = self.executor.loader.collect_sql([(plan, False)])

        self.stdout.write(self.style.MIGRATE_HEADING(f'App name: {app_label} - Migration: {migration_name}'))
        evaluator = SQLStatementsEvaluator(sql_statements)
        errors = evaluator.evaluate()

        for sql, error_info in errors:
            self.error = True
            self.stderr.write('    SQL is not secure to do without downtime.')
            self.stderr.write(f'    SQL sentence: {sql}', style_func=self.style.SQL_KEYWORD)
            self.stderr.write(f'    {error_info}', style_func=self.style.NOTICE)

        if not errors:
            self.stdout.write(self.style.SUCCESS('    Migration is secure.'))

    def get_app_name(self, filename) -> str:
        for app_name in self.loader.migrated_apps:
            path, _ = self.loader.migrations_module(app_name)
            path = path.replace('.', '/') + '/'
            if path in filename:
                return app_name

        raise ValueError('Cannot find the app name')



import pytest

from django_check_migration_wtf.evaluator import SQLRuleEvaluator
from django_check_migration_wtf.exceptions import SQLRuleError


class TestSQLRuleEvaluator(object):
    def test_evaluate_raise_sql_rule_error(self, dummy_sql_rule):
        evaluator = SQLRuleEvaluator()
        evaluator.rules.append(dummy_sql_rule)
        with pytest.raises(SQLRuleError, match=dummy_sql_rule.error_info):
            evaluator.evaluate('dummy')

    def test_evaluate_ok(self, dummy_sql_rule):
        evaluator = SQLRuleEvaluator()
        evaluator.rules.append(dummy_sql_rule)
        assert evaluator.evaluate('good') is None



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()

setup(
    name='django-check-migration-wtf',
    version='1.3.0',
    packages=find_packages(),
    description='A line of description',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Antonio Irizar',
    author_email='antonioirizar@gmail.com',
    url='https://github.com/antonioIrizar/django-check-migration-wtf',
    license='GLP3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
    ],
    keywords='django postgres postgresql migrations',
    python_requires='>=3.6',
    install_requires=[
        'Django>=2.2,<3.3',
        'PyGithub>=1.50,<2',
    ]
)



from unittest import mock

import pytest

from django_check_migration_wtf.github_migrations import GithubMigrations


class TestGithubMigrations:
    @pytest.mark.parametrize('filename',
                             ['/test/migrations/0001_32524.py',
                              'test/migrations/0001_32524.py',
                              'test2/test3/test/migrations/0001_32524.py', ]
                             )
    def test_get_migration_name(self, filename, github_migrations):
        assert github_migrations.get_migration_name(filename) == '0001_32524'

    @pytest.mark.parametrize('filename',
                             ['test/bad_migrations/0001_32524.py',
                              'test/migrations/0001_32524.py.back',
                              'test/migrations/0001_32524',
                              'test/migrations/__init__.py',
                              ]
                             )
    def test_get_migration_name_return_none(self, filename, github_migrations):
        assert github_migrations.get_migration_name(filename) is None

    @pytest.mark.parametrize(
        'status, filename, expected',
        [
            ('bad', '/test/migrations/0001_32524.py', [],),
            (GithubMigrations.FILE_STATUS_ADDED, '/test/migrations/bad', []),
            (GithubMigrations.FILE_STATUS_ADDED, '/test/migrations/0001_32524.py', [('/test/migrations/0001_32524.py', '0001_32524')]),
            (GithubMigrations.FILE_STATUS_MODIFIED, '/test/migrations/0001_32524.py', [('/test/migrations/0001_32524.py', '0001_32524')]),
            (GithubMigrations.FILE_STATUS_MODIFIED, '/test/migrations/__init__0001_32524.py', [('/test/migrations/__init__0001_32524.py', '__init__0001_32524')]),
        ]
    )
    def test_migration(self, status, filename, expected, github_migrations):
        github_migrations.comp = mock.PropertyMock(files=[
            mock.MagicMock(
                status=status,
                filename=filename,
            )
        ])

        assert github_migrations.migrations == expected
